import * as i0 from '@angular/core';
import { InjectionToken, Injectable, NgModule, importProvidersFrom } from '@angular/core';
import { setupListeners, QueryStatus, skipToken, defaultSerializeQueryArgs, buildCreateApi, coreModule, fetchBaseQuery as fetchBaseQuery$1 } from '@reduxjs/toolkit/query';
import { take, map, distinctUntilChanged, switchMap, tap, shareReplay, finalize } from 'rxjs/operators';
import * as i1 from '@ngrx/store';
import { provideState, createSelectorFactory, defaultMemoize } from '@ngrx/store';
import { isObservable, of, combineLatest, BehaviorSubject } from 'rxjs';
import { produceWithPatches } from 'immer';
import { __awaiter } from 'tslib';

let currentConfig;
const setupConfig = (config) => {
    currentConfig = config;
};
const defaultConfig = {
    setupListeners: false,
    baseUrl: '',
};
const STORE_RTK_QUERY_CONFIG = new InjectionToken('STORE_RTK_QUERY_CONFIG');

let service;
let delayedActions = [];
const internalBatchState = {
    currentSubscriptions: {},
};
function dispatch(action) {
    var _a, _b;
    if (typeof action === 'function') {
        return action(dispatch, getState, {});
    }
    else if (action.type.endsWith('internal_probeSubscription')) {
        const queryCacheKey = action.payload.queryCacheKey;
        const requestId = action.payload.requestId;
        const hasSubscription = !!((_a = internalBatchState.currentSubscriptions[queryCacheKey]) === null || _a === void 0 ? void 0 : _a[requestId]);
        return hasSubscription;
    }
    // Middleware dispatch actions before Store starts
    if (service && ((_b = Object.keys(getState())) === null || _b === void 0 ? void 0 : _b.length)) {
        delayedActions.map((delayedAction) => service.dispatch(delayedAction));
        delayedActions = [];
        service.dispatch(action);
    }
    else {
        delayedActions.push(action);
    }
    return action;
}
function getState() {
    return service === null || service === void 0 ? void 0 : service.getState();
}
function select(mapFn) {
    return service === null || service === void 0 ? void 0 : service.select(mapFn);
}
class ThunkService {
    constructor(store) {
        this.store = store;
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        service = this;
    }
    getState() {
        let state = {};
        // eslint-disable-next-line ngrx/no-store-subscription
        this.store.pipe(take(1)).subscribe((res) => (state = res));
        return state;
    }
    dispatch(action) {
        var _a;
        (_a = this.store) === null || _a === void 0 ? void 0 : _a.dispatch(action);
    }
    select(mapFn) {
        return this.store.select(mapFn);
    }
}
/** @nocollapse */ ThunkService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: ThunkService, deps: [{ token: i1.Store }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ ThunkService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: ThunkService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: ThunkService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i1.Store }]; } });

class StoreRtkQueryModule {
    constructor(thunkService) {
        this.thunkService = thunkService;
    }
    static forRoot(config) {
        const moduleConfig = Object.assign(Object.assign({}, defaultConfig), config);
        setupConfig(moduleConfig);
        if (moduleConfig.setupListeners) {
            setupListeners(dispatch, moduleConfig.customHandler);
        }
        return {
            ngModule: StoreRtkQueryModule,
            providers: [{ provide: STORE_RTK_QUERY_CONFIG, useValue: moduleConfig }, ThunkService],
        };
    }
}
/** @nocollapse */ StoreRtkQueryModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: StoreRtkQueryModule, deps: [{ token: ThunkService }], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ StoreRtkQueryModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.2.7", ngImport: i0, type: StoreRtkQueryModule });
/** @nocollapse */ StoreRtkQueryModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: StoreRtkQueryModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.2.7", ngImport: i0, type: StoreRtkQueryModule, decorators: [{
            type: NgModule,
            args: [{}]
        }], ctorParameters: function () { return [{ type: ThunkService }]; } });

function provideStoreRtkQuery(config) {
    return importProvidersFrom(StoreRtkQueryModule.forRoot(config));
}
function provideStoreApi(api) {
    return provideState(api.reducerPath, api.reducer, { metaReducers: [api.metareducer] });
}

const UNINITIALIZED_VALUE = Symbol();

function useStableQueryArgs(queryArgs, serialize, endpointDefinition, endpointName, cacheRef = {}) {
    const incoming = {
        queryArgs,
        serialized: typeof queryArgs == 'object' ? serialize({ queryArgs, endpointDefinition, endpointName }) : queryArgs,
    };
    if (!cacheRef.current || cacheRef.current.serialized !== incoming.serialized) {
        cacheRef.current = incoming;
    }
    return cacheRef.current.queryArgs;
}

function capitalize(str) {
    return str.replace(str[0], str[0].toUpperCase());
}

function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    }
    else {
        return x !== x && y !== y;
    }
}
function shallowEqual(objA, objB) {
    if (is(objA, objB)) {
        return true;
    }
    if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
        return false;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    // eslint-disable-next-line @typescript-eslint/prefer-for-of
    for (let i = 0; i < keysA.length; i++) {
        if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

function safeAssign(target, ...args) {
    Object.assign(target, ...args);
}

// const defaultQueryStateSelector: QueryStateSelector<any, any> = (x) => x;
const defaultMutationStateSelector = (x) => x;
/**
 * Wrapper around `defaultQueryStateSelector` to be used in `useQuery`.
 * We want the initial render to already come back with
 * `{ isUninitialized: false, isFetching: true, isLoading: true }`
 * to prevent that the library user has to do an additional check for `isUninitialized`/
 */
const noPendingQueryStateSelector = (selected) => {
    if (selected.isUninitialized) {
        return Object.assign(Object.assign({}, selected), { isUninitialized: false, isFetching: true, isLoading: selected.data !== undefined ? false : true, status: QueryStatus.pending });
    }
    return selected;
};
/**
 *
 * @param opts.api - An API with defined endpoints to create hooks for
 * @param opts.moduleOptions.useDispatch - The version of the `useDispatch` hook to be used
 * @param opts.moduleOptions.useSelector - The version of the `useSelector` hook to be used
 * @returns An object containing functions to generate hooks based on an endpoint
 */
function buildHooks({ api, moduleOptions: { useDispatch: dispatch, useSelector }, serializeQueryArgs, context, }) {
    return { buildQueryHooks, buildMutationHook, usePrefetch };
    function queryStatePreSelector(currentState, lastResult, queryArgs) {
        // if we had a last result and the current result is uninitialized,
        // we might have called `api.util.resetApiState`
        // in this case, reset the hook
        if ((lastResult === null || lastResult === void 0 ? void 0 : lastResult.endpointName) && currentState.isUninitialized) {
            const { endpointName } = lastResult;
            const endpointDefinition = context.endpointDefinitions[endpointName];
            if (serializeQueryArgs({
                queryArgs: lastResult.originalArgs,
                endpointDefinition,
                endpointName,
            }) ===
                serializeQueryArgs({
                    queryArgs,
                    endpointDefinition,
                    endpointName,
                }))
                lastResult = undefined;
        }
        // data is the last known good request result we have tracked
        // or if none has been tracked yet the last good result for the current args
        let data = currentState.isSuccess ? currentState.data : lastResult === null || lastResult === void 0 ? void 0 : lastResult.data;
        if (data === undefined)
            data = currentState.data;
        const hasData = data !== undefined;
        // isFetching = true any time a request is in flight
        const isFetching = currentState.isLoading;
        // isLoading = true only when loading while no data is present yet (initial load with no data in the cache)
        const isLoading = !hasData && isFetching;
        // isSuccess = true when data is present
        const isSuccess = currentState.isSuccess || (isFetching && hasData);
        return Object.assign(Object.assign({}, currentState), { data, currentData: currentState.data, isFetching,
            isLoading,
            isSuccess });
    }
    function usePrefetch(endpointName, defaultOptions) {
        return (arg, options) => dispatch(api.util.prefetch(endpointName, arg, Object.assign(Object.assign({}, defaultOptions), options)));
    }
    function buildQueryHooks(name) {
        const { initiate, select } = api.endpoints[name];
        const useQuerySubscription = (arg, { refetchOnReconnect, refetchOnFocus, refetchOnMountOrArgChange, skip = false, pollingInterval = 0 } = {}, promiseRef = {}, argCacheRef = {}, lastRenderHadSubscription = { current: false }) => {
            var _a, _b;
            const stableArg = useStableQueryArgs(skip ? skipToken : arg, 
            // Even if the user provided a per-endpoint `serializeQueryArgs` with
            // a consistent return value, _here_ we want to use the default behavior
            // so we can tell if _anything_ actually changed. Otherwise, we can end up
            // with a case where the query args did change but the serialization doesn't,
            // and then we never try to initiate a refetch.
            defaultSerializeQueryArgs, context.endpointDefinitions[name], name, argCacheRef);
            const subscriptionOptions = { refetchOnReconnect, refetchOnFocus, pollingInterval };
            const { queryCacheKey, requestId } = promiseRef.current || {};
            // HACK Because the latest state is in the middleware, we actually
            // dispatch an action that will be intercepted and returned.
            let currentRenderHasSubscription = false;
            if (queryCacheKey && requestId) {
                // This _should_ return a boolean, even if the types don't line up
                const returnedValue = dispatch(api.internalActions.internal_probeSubscription({
                    queryCacheKey,
                    requestId,
                }));
                if (process.env.NODE_ENV !== 'production' && process.env.NODE_ENV !== 'test') {
                    if (typeof returnedValue !== 'boolean') {
                        throw new Error(`Warning: Middleware for RTK-Query API at reducerPath "${api.reducerPath}" has not
              been added to the store. You must add the middleware for RTK-Query to function correctly!`);
                    }
                }
                currentRenderHasSubscription = !!returnedValue;
            }
            const subscriptionRemoved = !currentRenderHasSubscription && lastRenderHadSubscription.current;
            lastRenderHadSubscription.current = currentRenderHasSubscription;
            if (subscriptionRemoved) {
                (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                promiseRef.current = undefined;
            }
            if (stableArg !== skipToken) {
                const lastPromise = promiseRef === null || promiseRef === void 0 ? void 0 : promiseRef.current;
                const lastSubscriptionOptions = (_b = promiseRef.current) === null || _b === void 0 ? void 0 : _b.subscriptionOptions;
                if (!lastPromise || lastPromise.arg !== stableArg) {
                    lastPromise === null || lastPromise === void 0 ? void 0 : lastPromise.unsubscribe();
                    promiseRef.current = dispatch(initiate(stableArg, { subscriptionOptions, forceRefetch: refetchOnMountOrArgChange }));
                }
                else if (!shallowEqual(subscriptionOptions, lastSubscriptionOptions)) {
                    lastPromise.updateSubscriptionOptions(subscriptionOptions);
                }
            }
            return {
                /**
                 * A method to manually refetch data for the query
                 */
                refetch: () => {
                    var _a;
                    if (!promiseRef.current)
                        throw new Error('Cannot refetch a query that has not been started yet.');
                    return (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.refetch();
                },
            };
        };
        const useLazyQuerySubscription = ({ refetchOnReconnect, refetchOnFocus, pollingInterval = 0 } = {}, promiseRef = {}) => {
            var _a, _b;
            let argState = UNINITIALIZED_VALUE;
            const subscriptionOptions = { refetchOnReconnect, refetchOnFocus, pollingInterval };
            const lastSubscriptionOptions = (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.subscriptionOptions;
            if (!shallowEqual(subscriptionOptions, lastSubscriptionOptions)) {
                (_b = promiseRef.current) === null || _b === void 0 ? void 0 : _b.updateSubscriptionOptions(subscriptionOptions);
            }
            const trigger = (arg, { preferCacheValue = false } = {}) => {
                var _a;
                (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                const promise = dispatch(initiate(arg, { subscriptionOptions, forceRefetch: !preferCacheValue }));
                promiseRef.current = promise;
                argState = arg;
                return promise;
            };
            /* if "cleanup on unmount" was triggered from a fast refresh, we want to reinstate the query */
            if (argState !== UNINITIALIZED_VALUE && !promiseRef.current) {
                trigger(argState, { preferCacheValue: true });
            }
            return [trigger, argState];
        };
        const useQueryState = (arg, { skip = false, selectFromResult } = {}, lastValue = {}, argCacheRef = {}) => {
            const arg$ = isObservable(arg) ? arg : of(arg);
            return arg$.pipe(map((currentArg) => useStableQueryArgs(skip ? skipToken : currentArg, serializeQueryArgs, context.endpointDefinitions[name], name, argCacheRef)), distinctUntilChanged(shallowEqual), switchMap((stableArg) => {
                const selectDefaultResult = createSelectorFactory((projector) => defaultMemoize(projector, shallowEqual, shallowEqual))(select(stableArg), (subState) => queryStatePreSelector(subState, lastValue.current, stableArg));
                const querySelector = selectFromResult
                    ? createSelectorFactory((projector) => defaultMemoize(projector, shallowEqual, shallowEqual))(selectDefaultResult, selectFromResult)
                    : selectDefaultResult;
                return useSelector((state) => querySelector(state)).pipe(tap(() => (lastValue.current = selectDefaultResult(getState()))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }));
        };
        const useQuery = (arg, options) => {
            // Refs
            const promiseRef = {};
            const lastValue = {};
            const argRef = {};
            const arg$ = isObservable(arg) ? arg : of(arg);
            const options$ = isObservable(options) ? options : of(options);
            return combineLatest([arg$, options$.pipe(distinctUntilChanged((prev, curr) => shallowEqual(prev, curr)))]).pipe(switchMap(([currentArg, currentOptions]) => {
                const querySubscriptionResults = useQuerySubscription(currentArg, currentOptions, promiseRef, argRef);
                const queryStateResults$ = useQueryState(currentArg, Object.assign({ selectFromResult: currentArg === skipToken || (currentOptions === null || currentOptions === void 0 ? void 0 : currentOptions.skip) ? undefined : noPendingQueryStateSelector }, currentOptions), lastValue, argRef);
                return queryStateResults$.pipe(map((queryState) => (Object.assign(Object.assign({}, queryState), querySubscriptionResults))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }), finalize(() => {
                var _a;
                void ((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe());
                promiseRef.current = undefined;
            }));
        };
        const useLazyQuery = (options) => {
            // Refs
            const promiseRef = {};
            const lastValue = {};
            const triggerRef = {};
            const argRef = {};
            const infoSubject = new BehaviorSubject({ lastArg: UNINITIALIZED_VALUE });
            const info$ = infoSubject.asObservable();
            const options$ = isObservable(options) ? options : of(options);
            const state$ = combineLatest([
                options$.pipe(distinctUntilChanged((prev, curr) => shallowEqual(prev, curr)), tap((currentOptions) => {
                    const [trigger] = useLazyQuerySubscription(currentOptions, promiseRef);
                    triggerRef.current = trigger;
                })),
                info$.pipe(tap(({ lastArg, extra }) => {
                    var _a;
                    if (lastArg !== UNINITIALIZED_VALUE) {
                        (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.call(triggerRef, lastArg, extra);
                    }
                }), map(({ lastArg }) => lastArg), distinctUntilChanged(shallowEqual)),
            ]).pipe(switchMap(([currentOptions, currentArg]) => useQueryState(currentArg, Object.assign(Object.assign({}, currentOptions), { skip: currentArg === UNINITIALIZED_VALUE }), lastValue, argRef)), shareReplay({
                bufferSize: 1,
                refCount: true,
            }), finalize(() => {
                var _a;
                void ((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe());
                promiseRef.current = undefined;
            }));
            return {
                fetch: (arg, extra) => {
                    infoSubject.next({ lastArg: arg, extra });
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    return promiseRef.current;
                },
                state$,
                lastArg$: info$.pipe(map(({ lastArg }) => (lastArg !== UNINITIALIZED_VALUE ? lastArg : skipToken))),
            };
        };
        return {
            useQueryState,
            useQuerySubscription,
            useLazyQuerySubscription,
            useLazyQuery,
            useQuery,
            selector: select,
        };
    }
    function buildMutationHook(name) {
        const { initiate, select } = api.endpoints[name];
        const useMutation = ({ selectFromResult = defaultMutationStateSelector, fixedCacheKey } = {}) => {
            const promiseRef = {};
            const requestIdSubject = new BehaviorSubject(undefined);
            const requestId$ = requestIdSubject.asObservable();
            const triggerMutation = (arg) => {
                var _a;
                const promise = dispatch(initiate(arg, { fixedCacheKey }));
                if (!((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.arg.fixedCacheKey)) {
                    removePrevMutation();
                }
                promiseRef.current = promise;
                requestIdSubject.next(promise.requestId);
                return promise;
            };
            const reset = () => {
                removePrevMutation();
                requestIdSubject.next(undefined);
            };
            const removePrevMutation = () => {
                if (promiseRef.current) {
                    dispatch(api.internalActions.removeMutationResult({ requestId: promiseRef.current.requestId, fixedCacheKey }));
                    promiseRef.current = undefined;
                }
            };
            const state$ = requestId$.pipe(finalize(() => {
                var _a;
                (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.reset();
                promiseRef.current = undefined;
            }), distinctUntilChanged(shallowEqual), switchMap((requestId) => {
                var _a;
                const mutationSelector = createSelectorFactory((projector) => defaultMemoize(projector, shallowEqual, shallowEqual))(select(requestId ? { fixedCacheKey, requestId } : skipToken), (subState) => selectFromResult(subState));
                const currentState = useSelector((state) => mutationSelector(state));
                const originalArgs = fixedCacheKey == null ? (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.arg.originalArgs : undefined;
                return currentState.pipe(map((mutationState) => (Object.assign(Object.assign({}, mutationState), { originalArgs, reset }))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }));
            return { dispatch: triggerMutation, state$ };
        };
        return {
            useMutation,
            selector: select,
        };
    }
}

// Copied from https://github.com/feross/queue-microtask
let promise;
const queueMicrotaskShim = typeof queueMicrotask === 'function'
    ? queueMicrotask.bind(typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : globalThis)
    : // reuse resolved promise, and allocate it lazily
        (cb) => (promise || (promise = Promise.resolve())).then(cb).catch((err) => setTimeout(() => {
            throw err;
        }, 0));
const buildBatchedActionsHandler = ({ api, internalState }) => {
    const subscriptionsPrefix = `${api.reducerPath}/subscriptions`;
    let previousSubscriptions = null;
    let dispatchQueued = false;
    const { updateSubscriptionOptions, unsubscribeQueryResult } = api.internalActions;
    // Actually intentionally mutate the subscriptions state used in the middleware
    // This is done to speed up perf when loading many components
    const actuallyMutateSubscriptions = (mutableState, action) => {
        var _c, _d, _e, _f, _g, _h, _j;
        var _a, _b;
        if (updateSubscriptionOptions.match(action)) {
            const { queryCacheKey, requestId, options } = action.payload;
            if ((_c = mutableState === null || mutableState === void 0 ? void 0 : mutableState[queryCacheKey]) === null || _c === void 0 ? void 0 : _c[requestId]) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                mutableState[queryCacheKey][requestId] = options;
            }
            return true;
        }
        if (unsubscribeQueryResult.match(action)) {
            const { queryCacheKey, requestId } = action.payload;
            if (mutableState[queryCacheKey]) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                delete mutableState[queryCacheKey][requestId];
            }
            return true;
        }
        if (api.internalActions.removeQueryResult.match(action)) {
            delete mutableState[action.payload.queryCacheKey];
            return true;
        }
        if (action.type.endsWith('/pending')) {
            const { meta: { arg, requestId }, } = action;
            if (arg.subscribe) {
                const substate = ((_d = mutableState[_a = arg.queryCacheKey]) !== null && _d !== void 0 ? _d : (mutableState[_a] = {}));
                substate[requestId] = (_f = (_e = arg.subscriptionOptions) !== null && _e !== void 0 ? _e : substate[requestId]) !== null && _f !== void 0 ? _f : {};
                return true;
            }
        }
        if (action.type.endsWith('/rejected')) {
            const { meta: { condition, arg, requestId }, } = action;
            if (condition && arg.subscribe) {
                const substate = ((_g = mutableState[_b = arg.queryCacheKey]) !== null && _g !== void 0 ? _g : (mutableState[_b] = {}));
                substate[requestId] = (_j = (_h = arg.subscriptionOptions) !== null && _h !== void 0 ? _h : substate[requestId]) !== null && _j !== void 0 ? _j : {};
                return true;
            }
        }
        return false;
    };
    return (action, mwApi) => {
        var _c, _d;
        if (!previousSubscriptions) {
            // Initialize it the first time this handler runs
            previousSubscriptions = JSON.parse(JSON.stringify(internalState.currentSubscriptions));
        }
        // Intercept requests by hooks to see if they're subscribed
        // Necessary because we delay updating store state to the end of the tick
        if (api.internalActions.internal_probeSubscription.match(action)) {
            const { queryCacheKey, requestId } = action.payload;
            const hasSubscription = !!((_c = internalState.currentSubscriptions[queryCacheKey]) === null || _c === void 0 ? void 0 : _c[requestId]);
            return [false, hasSubscription];
        }
        // Update subscription data based on this action
        const didMutate = actuallyMutateSubscriptions(internalState.currentSubscriptions, action);
        if (didMutate) {
            if (!dispatchQueued) {
                queueMicrotaskShim(() => {
                    // Deep clone the current subscription data
                    const newSubscriptions = JSON.parse(JSON.stringify(internalState.currentSubscriptions));
                    // Figure out a smaller diff between original and current
                    const [, patches] = produceWithPatches(previousSubscriptions, () => newSubscriptions);
                    // Sync the store state for visibility
                    mwApi.next(api.internalActions.subscriptionsUpdated(patches));
                    // Save the cloned state for later reference
                    previousSubscriptions = newSubscriptions;
                    dispatchQueued = false;
                });
                dispatchQueued = true;
            }
            const isSubscriptionSliceAction = !!((_d = action.type) === null || _d === void 0 ? void 0 : _d.startsWith(subscriptionsPrefix));
            const isAdditionalSubscriptionAction = action.type.endsWith('/rejected') && action.meta.condition && !!action.meta.arg.subscribe;
            const actionShouldContinue = !isSubscriptionSliceAction && !isAdditionalSubscriptionAction;
            return [actionShouldContinue, false];
        }
        return [true, false];
    };
};

function buildMetaReducer({ api, moduleOptions: { useDispatch: dispatch }, }) {
    const anyApi = api;
    const reducerPath = anyApi.reducerPath;
    let nextState;
    const getState = () => nextState[reducerPath]
        ? nextState
        : // Query inside forFeature (Code splitting)
            { [reducerPath]: nextState };
    const middleware = anyApi.middleware({ dispatch, getState })(getState);
    const batchedActionsHandler = buildBatchedActionsHandler({ api, internalState: internalBatchState });
    return function (reducer) {
        return function (state, action) {
            nextState = reducer(state, action);
            if (action.type.startsWith(`${reducerPath}/`)) {
                batchedActionsHandler(action, { dispatch, getState, next: dispatch }, nextState);
            }
            middleware(action);
            return nextState;
        };
    };
}

function isQueryDefinition(e) {
    return e.type === 'query';
}
function isMutationDefinition(e) {
    return e.type === 'mutation';
}

const angularHooksModuleName = /* @__PURE__ */ Symbol();
/**
 * Creates a module that generates hooks from endpoints, for use with `buildCreateApi`.
 *
 * @returns A module for use with `buildCreateApi`
 */
const angularHooksModule = ({ useDispatch = dispatch, useSelector = select, getState: getState$1 = getState, } = {}) => ({
    name: angularHooksModuleName,
    init(api, { serializeQueryArgs }, context) {
        const anyApi = api;
        const { buildQueryHooks, buildMutationHook, usePrefetch } = buildHooks({
            api,
            moduleOptions: { useDispatch, useSelector, getState: getState$1 },
            serializeQueryArgs,
            context,
        });
        const metareducer = buildMetaReducer({ api, moduleOptions: { useDispatch } });
        safeAssign(anyApi, { usePrefetch });
        safeAssign(anyApi, { metareducer });
        return {
            injectEndpoint(endpointName, definition) {
                if (isQueryDefinition(definition)) {
                    const { useQuery, useLazyQuery, useLazyQuerySubscription, useQueryState, useQuerySubscription, selector } = buildQueryHooks(endpointName);
                    safeAssign(anyApi.endpoints[endpointName], {
                        useQuery,
                        useLazyQuery,
                        useLazyQuerySubscription,
                        useQueryState,
                        useQuerySubscription,
                        selector,
                    });
                    api[`use${capitalize(endpointName)}Query`] = useQuery;
                    api[`useLazy${capitalize(endpointName)}Query`] = useLazyQuery;
                }
                else if (isMutationDefinition(definition)) {
                    const { useMutation, selector } = buildMutationHook(endpointName);
                    safeAssign(anyApi.endpoints[endpointName], { useMutation, selector });
                    api[`use${capitalize(endpointName)}Mutation`] = useMutation;
                }
            },
        };
    },
});
const createApi = /* @__PURE__ */ buildCreateApi(coreModule(), angularHooksModule());

/**
 * This is a very small wrapper around fetch that aims to simplify requests.
 *
 * @example
 * ```ts
 * const baseQuery = fetchBaseQuery({
 *   baseUrl: 'https://api.your-really-great-app.com/v1/',
 *   prepareHeaders: (headers, { getState }) => {
 *     const token = (getState() as RootState).auth.token;
 *     // If we have a token set in state, let's assume that we should be passing it.
 *     if (token) {
 *       headers.set('authorization', `Bearer ${token}`);
 *     }
 *     return headers;
 *   },
 * })
 * ```
 *
 * @param baseUrl
 * The base URL for an API service.
 * Typically in the format of https://example.com/
 *
 * @param prepareHeaders
 * An optional function that can be used to inject headers on requests.
 * Provides a Headers object, as well as most of the `BaseQueryApi` (`dispatch` is not available).
 * Useful for setting authentication or headers that need to be set conditionally.
 *
 * @link https://developer.mozilla.org/en-US/docs/Web/API/Headers
 *
 * @param fetchFn
 * Accepts a custom `fetch` function if you do not want to use the default on the window.
 * Useful in SSR environments if you need to use a library such as `isomorphic-fetch` or `cross-fetch`
 *
 * @param paramsSerializer
 * An optional function that can be used to stringify querystring parameters.
 *
 * @param isJsonContentType
 * An optional predicate function to determine if `JSON.stringify()` should be called on the `body` arg of `FetchArgs`
 *
 * @param jsonContentType Used when automatically setting the content-type header for a request with a jsonifiable
 * body that does not have an explicit content-type header. Defaults to `application/json`.
 *
 * @param jsonReplacer Custom replacer function used when calling `JSON.stringify()`.
 *
 * @param timeout
 * A number in milliseconds that represents the maximum time a request can take before timing out.
 */
function fetchBaseQuery(queryArgs = {}) {
    const baseQueryEnvironment = fetchBaseQuery$1(queryArgs);
    return (args, api, extraOptions) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const baseUrl = (_a = currentConfig === null || currentConfig === void 0 ? void 0 : currentConfig.baseUrl) !== null && _a !== void 0 ? _a : '';
        const argUrl = typeof args === 'string' ? args : args.url;
        const url = `${baseUrl}${argUrl}`;
        const newArgs = typeof args === 'string' ? url : Object.assign(Object.assign({}, args), { url });
        return yield baseQueryEnvironment(newArgs, api, extraOptions);
    });
}

/*
 * Public API Surface of ngrx-rtk-query
 */

/**
 * Generated bundle index. Do not edit.
 */

export { StoreRtkQueryModule, ThunkService, angularHooksModule, createApi, dispatch, fetchBaseQuery, provideStoreApi, provideStoreRtkQuery };
//# sourceMappingURL=ngrx-rtk-query.mjs.map
